﻿[CmdletBinding()]
param 
(
    [Parameter(Mandatory=$true)]
    [string] $subscription,

    [Parameter(Mandatory=$true)]
    [string] $vaultName
)

# prepare files structure
$targetFolder = Join-Path -Path $PSScriptRoot -ChildPath keyvault-backup
$secretsPath = Join-Path -Path $targetFolder secrets
$certificatesPath = Join-Path -Path $targetFolder certificates

if (Test-Path $targetFolder) {
    Remove-Item -Path $targetFolder -Recurse -Force
}

New-Item -ItemType directory -Path $targetFolder -Name secrets
New-Item -ItemType directory -Path $targetFolder -Name certificates

# connect to azure
Connect-AzAccount
Select-AzSubscription -SubscriptionId $subscription

# get user running the script
$context = Get-AzContext
$user = Get-AzADUser -UserPrincipalName $context.Account.Id

# ensure 'backup' and 'list' permissions for keyVault secrets and certificates
$vault = Get-AzKeyVault -VaultName $vaultName
$userPermissions = $vault.AccessPolicies | Where-Object { $_.ObjectId -eq $user.Id}
$permissionsToSecrets = New-Object Collections.Generic.List[string]
$permissionsToCert = New-Object Collections.Generic.List[string]

if ($userPermissions) {
    $permissionsToSecrets.AddRange($userPermissions.PermissionsToSecrets)
    $permissionsToCert.AddRange($userPermissions.PermissionsToCertificates)
}

if (!$permissionsToSecrets.Contains("backup")) {
    $permissionsToSecrets.Add("backup")
}

if (!$permissionsToSecrets.Contains("list")) {
    $permissionsToSecrets.Add("list")
}

if (!$permissionsToCert.Contains("backup")) {
    $permissionsToCert.Add("backup")
}

if (!$permissionsToCert.Contains("list")) {
    $permissionsToCert.Add("list")
}

Set-AzKeyVaultAccessPolicy -VaultName $vaultName -ObjectId $user.Id -PermissionsToSecrets $permissionsToSecrets -PermissionsToCertificates $permissionsToCert

# backup secrets
Get-AzKeyVaultSecret -VaultName $vaultName -WarningAction Ignore | Where-Object { $_.ContentType -ne 'application/x-pkcs12' } | ForEach-Object {
    Write-Host 'Processing secret: ' $_.Name
    $itemPath = Join-Path -Path $secretsPath -ChildPath $_.Name
    Backup-AzKeyVaultSecret -VaultName $vaultName -Name $_.Name -OutputFile $itemPath -Force
}

# backup certificates
Get-AzKeyVaultCertificate -VaultName $vaultName | ForEach-Object {
    Write-Host 'Processing certificate: ' $_.Name
    $itemPath = Join-Path -Path $certificatesPath -ChildPath $_.Name
    Backup-AzKeyVaultCertificate -VaultName $vaultName -Name $_.Name -OutputFile $itemPath -Force
}

# zip files
$zipPath = Join-Path $PSScriptRoot keyvault-backup.zip
Compress-Archive -Path (Join-Path $targetFolder *) -DestinationPath $zipPath -Force
Remove-Item $targetFolder -Recurse -Force