﻿[CmdletBinding()]
param 
(
    [Parameter(Mandatory=$true)]
    [string] $subscription,

    [Parameter(Mandatory=$true)]
    [string] $vaultName,
    
    [string] $sourceFile = "keyvault-backup.zip"
)

# prepare files structure
$targetFolder = Join-Path -Path $PSScriptRoot -ChildPath keyvault-backup
$secretsPath = Join-Path -Path $targetFolder secrets
$certificatesPath = Join-Path -Path $targetFolder certificates

if (Test-Path $targetFolder) {
    Remove-Item -Path $targetFolder -Recurse -Force
}

New-Item -ItemType directory -Path $targetFolder

# extract source zip
Expand-Archive -Path $sourceFile -DestinationPath $targetFolder -ErrorAction Stop

# connect to azure
Connect-AzAccount
Select-AzSubscription -SubscriptionId $subscription -ErrorAction Stop

# get user running the script
$context = Get-AzContext -ErrorAction Stop
$user = Get-AzADUser -UserPrincipalName $context.Account.Id -ErrorAction Stop

# ensure 'restore' and 'get' permissions for keyVault secrets and certificates
$vault = Get-AzKeyVault -VaultName $vaultName -ErrorAction Stop
$userPermissions = $vault.AccessPolicies | Where-Object { $_.ObjectId -eq $user.Id}
$permissionsToSecrets = New-Object Collections.Generic.List[string]
$permissionsToCert = New-Object Collections.Generic.List[string]

if ($userPermissions) {
    $permissionsToSecrets.AddRange($userPermissions.PermissionsToSecrets)
    $permissionsToCert.AddRange($userPermissions.PermissionsToCertificates)
}

if (!$permissionsToSecrets.Contains("restore")) {
    $permissionsToSecrets.Add("restore")
}

if (!$permissionsToSecrets.Contains("get")) {
    $permissionsToSecrets.Add("get")
}

if (!$permissionsToCert.Contains("restore")) {
    $permissionsToCert.Add("restore")
}

if (!$permissionsToCert.Contains("get")) {
    $permissionsToCert.Add("get")
}

Set-AzKeyVaultAccessPolicy -VaultName $vaultName -ObjectId $user.Id -PermissionsToSecrets $permissionsToSecrets -PermissionsToCertificates $permissionsToCert -ErrorAction Stop

# restore secrets
Get-ChildItem -Path $secretsPath | ForEach-Object {
    
    $secret = Get-AzKeyVaultSecret -VaultName $vaultName -Name $_.Name -WarningAction Ignore
        
    if ($secret -eq $null) {
        
        Write-Host "Restore secret $($_.Name)"
                
        Restore-AzKeyVaultSecret -VaultName $vaultName -InputFile $_.FullName -WarningAction Ignore
    
    } else {        
        Write-Host "Skip secret $($_.Name) because already exists"
    }
}

# restore certificates
Get-ChildItem -Path $certificatesPath | ForEach-Object {

    $cert = Get-AzKeyVaultCertificate -VaultName $vaultName -Name $_.Name
        
    if ($cert -eq $null) {

        Write-Host "Restore certificate $($_.Name)"
        
        Restore-AzKeyVaultCertificate -VaultName $vaultName -InputFile $_.FullName
    
    } else {
        Write-Host "Skip certificate $($_.Name) because already exists"
    }
}

Remove-Item -Path $targetFolder -Recurse -Force